import re
from coastal_resilience_utilities.utils.dataset import meters_to_degrees

VARS = [
    "WaterDepthMax",
    "Exposure", 
    "PercentDamage",
    "DDE", 
    "Damages", 
    "Population"
]
CLIMATES = ["Historic", "Future2050"]
SCENARIOS = ["S1", "S2", "S3", "S4"]
RPS = ["Tr10", "Tr25", "Tr50", "Tr100"]

FORMATTER = lambda v, c, s, r : f"{v}_{c}_{s}_{r}"

def get_all(regex=r'.*'):
    from itertools import product
    return [
        i for i in [FORMATTER(v, c, s, r) for v, c, s, r in product(VARS, CLIMATES, SCENARIOS, RPS)]
        if re.match(regex, i)
    ]
    
# Example dictionary
caribbean_config = {
    "id": "caribbean",
    "bounds": (
        -99.90511627,   
        6.42762405, 
        -58.32889149,  
        32.70681188
    ),
    "epsg": "4326",
    "dx": meters_to_degrees(30, 10),
    "chunk_size": 1000,
    "varnames": get_all(),
    "storage": '/app/data/caribbean-test.zarr'
}

dr_config = dict(   
    id = "hispaniola",
    bounds = (
        -74.8310,
        16.5424,
        -68.0246,
        20.3981,
    ),
    epsg = "4326",
    dx = meters_to_degrees(30, 10),
    chunk_size = 1000,
    varnames = get_all(),
    storage = '/app/data/hispaniola-test.zarr',
)





import zarr
from pathlib import Path

class BaseStorage:
    def __init__(self):
        pass
    
    def get_storage(self):
        pass
    
    def get_root_group(self):
        pass
    
    def create_dataset(self, shape, group=None, varnames=None):
        pass


class ArrayLakeStorage(BaseStorage):
    def __init__(self, client: str, repo: str, disk_store: str):
        self.client = client
        self.repo = repo
        self.disk_store = disk_store
        
    def get_storage(self):
        return self.repo.store
    
    @property
    def root_group(self):
        return self.repo.root_group
    
    def create_group(self, group: str):
        self.root_group.create_group(group)
        
    def get_group(self, group: str = None):
        return self.root_group[group]
    
    def delete_group(self, group: str):
        del self.root_group[group]
        
    def create_dataset(self, var, group=None, varnames=None):
        pass

class IceChunkLocalDatastore(BaseStorage):
    
    def __init__(self, path: str, mode: str = "w"):
        storage_config = icechunk.StorageConfig.filesystem(path)
        try:
            self.store = icechunk.IcechunkStore.create(storage_config, mode=mode)
        except ValueError:
            self.store = icechunk.IcechunkStore.open_existing(
                storage=storage_config,
                mode="r+"
            )

    def get_storage(self):
        return self.store
    
    @property
    def root_group(self):
        return zarr.group(store=self.store)
    
    def create_group(self, group: str):
        self.root_group.create_group(group)
        
    def get_group(self, group: str = None):
        return self.root_group[group]
    
    def delete_group(self, group: str):
        del self.root_group[group]
        
    def create_dataset(self, var, group=None, varnames=None):
        pass
    
    
class DummyRepo:
    def commit(self, message: str):
        pass

class PlainOlZarrStore(BaseStorage):
    def __init__(self, path: str):
        self.store = zarr.storage.DirectoryStore(Path(path) / 'data.zarr')
        self.repo = DummyRepo()
        
    def get_storage(self):
        return self.store
    
    @property
    def root_group(self):
        return zarr.group(store=self.store)
    
    def create_group(self, group: str):
        self.root_group.create_group(group)
        
    def get_group(self, group: str = None):
        return self.root_group[group]
    
    def delete_group(self, group: str):
        del self.root_group[group]
        
    def create_dataset(self, var, group=None, varnames=None):
        pass


from setuptools import setup, find_packages
import os

def parse_requirements(filename):
    with open(filename, 'r') as file:
        return [line.strip() for line in file if line.strip() and not line.startswith('#')]

requirements_file = os.environ.get('REQUIREMENTS_FILE', 'requirements-freeze.txt')
version = os.environ.get('VERSION', '0.1.4')

setup(
    name='crl_datacube',
    version=version,
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['**/*.csv']},
    install_requires=parse_requirements(requirements_file),
    author='Chris Lowrie',
    author_email='chlowrie@ucsc.edu',
    description='Utilities for scaling geospatial analyses',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://git.ucsc.edu/chlowrie/datacube',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.12.3',
)